local L = AceLibrary("AceLocale-2.2"):new("Fence_AutoFill")
local l = AceLibrary("AceLocale-2.2"):new("Fence")

-- Localization
L:RegisterTranslations("enUS", function() 
	return {
		["Auto-Fill"] = true,
		["Auction Runtime"] = true,
		["Set default auction runtime."] = true,
		["Remember last used auction runtime."] = true,
		["%s now set to %s hours."] = true,
		["Type in 2, 8 or 24"] = true,
		["Duration"] = true,
		["Auto-Buyout"] = true,
		["Markup"] = true,
		["Set markup to a number between 1 and 1000"] = true,
		["Automatically fills the buyout price * customizable markup when no price informations are available."] = true
	}
end)

L:RegisterTranslations("deDE", function() 
	return {
		["Auto-Fill"] = "Auto-Ausf\195\188llen",
		["Auction Runtime"] = "Auktionslaufzeit",
		["Set default auction runtime."] = "Legt Standard-Auktionslaufzeit fest.",
		["Remember last used auction runtime."] = "Merkt sich zuletzt benutzte Auktionslaufzeit.",
		["%s now set to %s hours."] = "%s auf %s Stunden ge\195\164ndert.",
		["Type in 2, 8 or 24"] = "2, 8 oder 24 eingeben",
		["Duration"] = "Dauer",
		["Auto-Buyout"] = "Auto-Sofortkauf",
		["Markup"] = "Aufschlag",
		["Set markup to a number between 1 and 1000"] = "Wert f\195\188r Aufschlag zwischen 1 und 1000 festlegen.",
		["Automatically fills the buyout price * customizable markup when no price informations are available."] = 
			"F\195\188llt automatisch den Sofortkauf + festgelegten Aufschlag aus, wenn keine Preisinformationen vorliegen."
	}
end)

Fence_AutoFill = Fence:NewModule("AutoFill")

	function Fence_AutoFill:OnInitialize()
		self.db = Fence:AcquireDBNamespace("AutoFill")
		if not self.db.profile.prices then self.db.profile.prices = {} end
		if not self.db.profile.lastduration then self.db.profile.lastduration = 480 end
		if not self.db.profile.markup then self.db.profile.markup = 1.5 end
		if self.db.profile.autobuy == nil then self.db.profile.autobuy = true end

		
		Fence.options.args.autofill = {
			type = 'group',
			name = L["Auto-Fill"], aliases = "af",
			desc = string.format(l["%s options"], L["Auto-Fill"]),
				args = {
					toggle = {
						type = 'toggle',
						name = L["Auto-Fill"],
						desc = string.format(l["Toggles %s function."], L["Auto-Fill"]),
						get = function() return Fence:IsModuleActive("AutoFill") end,
						set = function(v) Fence:ToggleModuleActive("AutoFill", v) end
					},
					duration = {
						type = 'text',
						name = L["Duration"], aliases = "dur",
						desc = L["Set default auction runtime."],
						usage = L["Type in 2, 8 or 24"],
						message = L["%s now set to %s hours."],
						get	= function() return self.db.profile.dur end,
						set =	function(v) self.db.profile.dur = v end,
						validate = {2, 8, 24}
					},
					last = {
						type = 'toggle',
						name = L["Auction Runtime"],
						desc = L["Remember last used auction runtime."],
						get	= function() return self.db.profile.last end,
						set =	function(v) self.db.profile.last = v end
					},
					autobuyout = {
						type = 'group',
						name = L["Auto-Buyout"], aliases = "ab",
						desc = L["Automatically fills the buyout price * customizable markup when no price informations are available."],
							args = {
								toggle = {
									type = 'toggle',
									name = L["Auto-Buyout"],
									desc = string.format(l["Toggles %s function."], L["Auto-Buyout"]),
									get	= function() return self.db.profile.autobuy end,
									set =	function(v) self.db.profile.autobuy = v end
								},
								markup = {
									type = 'range', aliases = "mu",
									name = L["Markup"],
									desc = L["Set markup to a number between 1 and 1000"],
									get	= function() return self.db.profile.markup end,
									set =	function(v) self.db.profile.markup = v end,
									min = 1,
									max = 1000
								}
							}
					}
				}
		}
	end
	
	function Fence_AutoFill:OnEnable()
		self:RegisterEvent("AH_LOADED")
		self:RegisterEvent("AUCTION_OWNED_LIST_UPDATE")
	end
	
	function Fence_AutoFill:AH_LOADED()
		self:Hook("StartAuction", true)
		self:Hook("AuctionSellItemButton_OnEvent", true)
		self:UnregisterEvent("AH_LOADED")
	end

	function Fence_AutoFill:OnDisable()
		if self:IsHooked("StartAuction") then self:Unhook("StartAuction") end
		if self:IsHooked("AuctionSellItemButton_OnEvent") then self:Unhook("AuctionSellItemButton_OnEvent") end
	end

	function Fence_AutoFill:AUCTION_OWNED_LIST_UPDATE()
		local d, sa, ma, la, du
		if self.db.profile.last then
			du = self.db.profile.lastduration
			if du == 120 then 
				sa, ma, la  = 1, 0, 0
			elseif du == 480 then
				sa, ma, la = 0, 1, 0
			elseif du == 1440 then
				sa, ma, la  = 0, 0, 1
			else
				return
			end
			self:SetDur(sa, ma, la, du) 
			return
		end
		if self.db.profile.dur then
			d = tonumber(self.db.profile.dur)
			if d == 2 then
				sa, ma, la, du  = 1, 0, 0, 120
			elseif d == 8 then
				sa, ma, la, du  = 0, 1, 0, 480
			elseif d == 24 then
				sa, ma, la, du  = 0, 0, 1, 1440
			else 
				return
			end
			self:SetDur(sa, ma, la, du) 
			return
		end
	end
	
	function Fence_AutoFill:SetDur(sa, ma, la, du)
		AuctionsShortAuctionButton:SetChecked(sa)
		AuctionsMediumAuctionButton:SetChecked(ma)
		AuctionsLongAuctionButton:SetChecked(la)
		AuctionFrameAuctions.duration = du
	end
	
	function Fence_AutoFill:AuctionSellItemButton_OnEvent()
		self.hooks["AuctionSellItemButton_OnEvent"]()
		local startPrice = MoneyInputFrame_GetCopper(StartPrice)
		local name, texture, count, quality, canUse, price = GetAuctionSellItemInfo()
--~ 		self:Print(name,startPrice)
		local db = self.db.profile.prices[name]
		if not db and self.db.profile.autobuy then 
			MoneyInputFrame_SetCopper(BuyoutPrice, max(100, floor(startPrice * self.db.profile.markup))) 
				return
		end
		MoneyInputFrame_SetCopper(StartPrice, db.s * count)
		MoneyInputFrame_SetCopper(BuyoutPrice, db.b * count)
	end
	
	function Fence_AutoFill:StartAuction(start, buyout, duration)
		local name, texture, count, quality, canUse, price = GetAuctionSellItemInfo()
		local db = self.db.profile.prices[name] or {}
		db.s = start  / count
		db.b = buyout / count
		self.db.profile.prices[name] = db
		if self.db.profile.last then self.db.profile.lastduration = duration end
		self.hooks["StartAuction"](start, buyout, duration)
	end