-- Credits: phyber
Fence_Bookmarks = Fence:NewModule("Bookmarks")
local _G = getfenv(0)
local dewdrop = AceLibrary("Dewdrop-2.0")
local L = AceLibrary("AceLocale-2.2"):new("Bookmarks")
L:RegisterTranslations("enUS", function() return {
	["Bookmarks"] = true,
	["Bookmark Current Search"] = true,
	["Add Bookmark"] = true,
	["Delete Bookmark"] = true,
} end)

function Fence_Bookmarks:OnInitialize()
	self.db = Fence:AcquireDBNamespace("Bookmarks")
	if not self.db.profile.bookmarks then self.db.profile.bookmarks = {} end
	self.menu = {
		desc = L["Bookmarks"],
		type = 'group',
		args = {
			addbookmark = {
				desc = L["Add Bookmark"],
				name = L["Add Bookmark"],
				type = 'text',
				get = false,
				set = function(v)
					self.db.profile.bookmarks[v] = true
					dewdrop:Refresh()
				end,
				validate = function(v)
					return string.find(v, "^%w+$")
				end
			},
			delbookmark = {
				desc = L["Delete Bookmark"],
				name = L["Delete Bookmark"],
				type = 'text',
				get = false,
				set = function(v)
					self.db.profile.bookmarks[v] = nil
					self:UpdateValidateList()
					dewdrop:Refresh()
				end,
				validate = self.db.profile.bookmarks
			}
		}
	}
end

function Fence_Bookmarks:OnEnable()
	self:RegisterEvent("AH_OPENED", "CreateButton")
end

function Fence_Bookmarks:CreateButton()
	local BookmarkButton = CreateFrame("Button", "BookmarkButton", AuctionFrameBrowse)
	BookmarkButton:SetWidth(24)
	BookmarkButton:SetHeight(24)
	BookmarkButton:SetPoint("RIGHT", "BrowseName", "RIGHT", 6, 0)
	BookmarkButton:SetNormalTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Up")
	BookmarkButton:SetPushedTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Down")
	BookmarkButton:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight")
	BookmarkButton:SetScript("OnClick", function()
		self:ToggleDropdown()
	end)

	dewdrop:Register(BookmarkButton, 'children', function() self:ToggleDropdown() end, 'dontHook', true)

	self:UnregisterEvent("AH_OPENED")
end

function Fence_Bookmarks:UpdateValidateList()
	for item,_ in pairs(self.db.profile.bookmarks) do 
		table.insert(self.menu.args.delbookmark.validate, item)
	end
end

function Fence_Bookmarks:SetSearch(item)
	BrowseName:SetText(item)
	if CanSendAuctionQuery() then
		AuctionFrameBrowse_Search()
	end
end

function Fence_Bookmarks:BookmarkCurrent()
	local curSearch = BrowseName:GetText()
	if curSearch then
		self.db.profile.bookmarks[curSearch] = true
	end
end

function Fence_Bookmarks:ToggleDropdown()
	if not dewdrop:IsOpen(BookmarkButton) then
		dewdrop:AddLine(
			'text', L["Bookmarks"],
			'isTitle', true,
			'justifyH', "CENTER",
			'notCheckable', true,
			'textHeight', 14,
			'textR', 1,
			'textG', 1,
			'textB', 1
		)

		for item, _ in pairs(self.db.profile.bookmarks) do
			dewdrop:AddLine(
				'text', item,
				'textHeight', 12,
				'func', "SetSearch",
				'arg1', item
			)
		end

		-- spacer
		dewdrop:AddLine()

		-- bookmark current search
		--[[dewdrop:AddLine(
			'text', L["Bookmark Current Search"],
			'textHeight', 12,
			'func', "BookmarkCurrent"
		)

		-- add bookmark manually
		dewdrop:AddLine(

		)

		-- delete bookmark
		dewdrop:AddLine(
			'text', L["Delete Bookmark"],
			'textHeight', 12,
			'func', function()
			end
		)]]
		dewdrop:InjectAceOptionsTable(self, self.menu)
	else
		dewdrop:Close(BookmarkButton)
	end
end
